@@ -1,7 +1,7 @@
 /*
    Kickshaw - A Menu Editor for Openbox
 
-   Copyright (c) 2010–2022        Marcus Schätzle
+   Copyright (c) 2010–2024        Marcus Schätzle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,9 +20,18 @@
 #ifndef __declarations_definitions_and_enumerations_h
 #define __declarations_definitions_and_enumerations_h
 
-#define PCRE2_CODE_UNIT_WIDTH 8
-#include <pcre2.h>
+// The following pragmas also work with Clang.
+#if !(GLIB_CHECK_VERSION(2,44,0))
+#pragma GCC error "The minimum version of GLib needed to compile Kickshaw is 2.44."
+#endif
+#if !(GTK_CHECK_VERSION(3,18,0))
+#pragma GCC error "The minimum version of GTK needed to compile Kickshaw is 3.18."
+#endif
 
+#define G_DISABLE_ASSERT // Leave this uncommented when compiling after testing
+
+#include <glib/gi18n.h>
+#include <libintl.h>
 #include <stdbool.h>
 
 
@@ -48,6 +57,9 @@ void single_field_entry (void);
 
 // auxiliary.c
 
+#if !__GLIBC__
+gint8 strverscmp_nonglibc (const gchar *version1, const gchar *version2);
+#endif
 void add_elements_to_ptr_array (GPtrArray *array, ...) G_GNUC_NULL_TERMINATED;
 gboolean add_icon_occurrence_to_list (G_GNUC_UNUSED GtkTreeModel *foreach_model, 
                                                     GtkTreePath  *foreach_path,
@@ -76,7 +88,6 @@ GtkWidget *create_dialog (      GtkWidget **dialog,
                           const gchar      *button_txt_1, 
                           const gchar      *button_txt_2, 
                           const gchar      *button_txt_3, 
-                          const gboolean    use_header_bar, 
                           const gboolean    show_immediately);
 void create_file_dialog (      GtkWidget **dialog, 
                          const gboolean    open);
@@ -95,7 +106,6 @@ void find_in_columns_management (const gboolean    origin_is_fr,
 void free_elements_of_static_string_array (      gchar    **string_array, 
                                                  gint8      number_of_fields, 
                                            const gboolean   set_to_NULL);
-guint32 get_compile_options (const gboolean find_and_replace);
 gchar *get_font_name (void);
 guint get_font_size (void);
 gchar *get_modification_time_for_icon (const gchar *icon_path);
@@ -103,12 +113,9 @@ gchar *get_txt_from_resources (      GResource *resources,
                                const gchar     *path);
 void get_toplevel_iter_from_path (GtkTreeIter *local_iter, 
                                   GtkTreePath *local_path);
+const gchar *installed_or_local (void);
 GtkWidget *new_label_with_formattings (const gchar    *label_txt, 
                                        const gboolean  wrap);
-gboolean regex_match_simple (const gchar   *regex_str, 
-                             const gchar   *string, 
-                             const guint32  compile_options, 
-                             const guint32  match_options);
 void repopulate_txt_fields_array (void);
 gboolean rm_rf (GFile         *file, 
                 GCancellable  *cancellable, 
@@ -210,6 +217,9 @@ gboolean close_window (GtkWidget   *window,
                        gpointer     keys);
 gboolean continue_despite_unsaved_changes (void);
 PangoTabArray *create_and_set_tab_array (void);
+void get_latest_version_str_from_server_callback (              GObject      *source_object, 
+                                                                GAsyncResult *result, 
+                                                  G_GNUC_UNUSED gpointer      user_data);
 gboolean selection_block_unblock (G_GNUC_UNUSED GtkTreeSelection *selection, 
                                   G_GNUC_UNUSED GtkTreeModel     *model,
                                   G_GNUC_UNUSED GtkTreePath      *path, 
@@ -265,6 +275,9 @@ enum { MENU, PIPE_MENU, ITEM, SEPARATOR, ACTION_OR_OPTION, NUMBER_OF_ADD_BUTTONS
 enum { NONE_OR_VISIBLE_ANCESTOR, INVISIBLE_ANCESTOR, INVISIBLE_ORPHANED_ANCESTOR };
 // columns
 enum { COL_MENU_ELEMENT, COL_TYPE, COL_VALUE, COL_MENU_ID, COL_EXECUTE, COL_ELEMENT_VISIBILITY, NUMBER_OF_COLUMNS };
+// visibility states
+enum { VISIBLE, INVISIBLE_MENU, INVISIBLE_ITEM, INVISIBLE_ORPHANED_MENU, 
+       INVISIBLE_DSCT_OF_INVISIBLE_MENU, INVISIBLE_DSCT_OF_INVISIBILE_ORPHANED_MENU, NUMBER_OF_VISIBILITY_STATES };
 // entry fields
 enum { MENU_ELEMENT_OR_VALUE_ENTRY, ICON_PATH_ENTRY, MENU_ID_ENTRY, EXECUTE_ENTRY, NUMBER_OF_ENTRY_FIELDS };
 // execute options
@@ -320,8 +333,16 @@ enum { TXT_RENDERER, EXCL_TXT_RENDERER, PIXBUF_RENDERER, BOOL_RENDERER, NUMBER_O
 enum { MENU_FILE_HINTS, WORKING_WITH_KS_HINTS, MISCELLANEOUS_HINTS, NUMBER_OF_HINT_TXTS };
 // for create_file_dialog ()
 enum { SAVE_FILE, OPEN_FILE };
-// for regex compilation
-enum { FIND, FIND_AND_REPLACE };
+// for write_settings ()
+enum { WRITE_CURRENT_STATES, CHANGE_STATES_TO_DEFAULT };
+// for the buttons inside the "new xy" mask
+enum { CANCEL_NEW_XY, RESET_NEW_XY, OK_NEW_XY };
+// for push_new_item_on_undo_stack ()
+enum { NOT_AFTER_SAVE, AFTER_SAVE };
+// for create_dialog ()
+enum { DONT_SHOW_IMMEDIATELY, SHOW_IMMEDIATELY };
+// for iterations through the tree
+enum { CONTINUE_ITERATING, STOP_ITERATING };
 
 
 /*
@@ -331,14 +352,14 @@ enum { FIND, FIND_AND_REPLACE };
 */
 
 
-#define KICKSHAW_VERSION "1.0.73"
-#define RELEASED_IN "2022"
+#define KICKSHAW_VERSION "1.1.41"
+#define RELEASED_IN "2024"
 
 #define TREEVIEW_COLUMN_OFFSET NUMBER_OF_TS_ELEMENTS - NUMBER_OF_COLUMNS
 #define FREE_AND_REASSIGN(string, new_value) { g_free (string); string = new_value; }
 #define NOT_NULL_AND_NOT_EMPTY(string) (string && *string)
 #define STREQ(string1, string2) (g_strcmp0 ((string1), (string2)) == 0)
-#define FOREACH_IN_ARRAY(array, counter) for (counter = 0; counter < sizeof (array) / sizeof *(array); counter++)
+#define FOREACH_IN_ARRAY(array, counter) for (counter = 0; counter < sizeof (array) / sizeof *(array); ++counter)
 #define FOREACH_IN_LIST(list, counter) for (counter = list; counter != NULL; counter = counter->next)
 
 #if !(GLIB_CHECK_VERSION(2,70,0))
@@ -347,8 +368,6 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoLayout, g_object_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(PangoTabArray, pango_tab_array_free)
 #endif
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(cairo_pattern_t, cairo_pattern_destroy)
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(pcre2_code, pcre2_code_free)
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(pcre2_match_data, pcre2_match_data_free)
 #if !(GTK_CHECK_VERSION(3,24,0))
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GResource, g_resource_unref)
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(GtkTreePath, gtk_tree_path_free)
@@ -383,6 +402,8 @@ typedef struct {
 } b_KickshawSettings;
 
 typedef struct {
+    const gchar *locale;
+
     GtkApplication *app;
 
     GtkWidget *window;
@@ -498,6 +519,10 @@ typedef struct {
     GtkListStore *liststore;
     GtkTreeModel *ls_model;
 
+    GtkWidget *about;
+    gchar *version_info;
+    gboolean stopped_trying_to_connect_to_server;
+
     gchar *settings_file_actions;
     GPtrArray *settings_menu_items;
 
@@ -520,6 +545,7 @@ typedef struct {
 
     GdkPixbuf *invalid_icon_imgs[NUMBER_OF_INVALID_ICON_IMGS];
 
+    const gchar *home_dir;
     gchar *filename;
 
     gchar *tmp_path;
