@@ -1,7 +1,7 @@
 /*
    Kickshaw - A Menu Editor for Openbox
 
-   Copyright (c) 2010–2022        Marcus Schätzle
+   Copyright (c) 2010–2024        Marcus Schätzle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -97,7 +97,7 @@ static void sort_execute_or_startupnotify_options (      GtkTreeIter *parent,
                 break;
             }
             else {
-                for (gint follow_ch_cnt = ch_cnt + 1; follow_ch_cnt < gtk_tree_model_iter_n_children (ks.ts_model, parent); follow_ch_cnt++) {
+                for (gint follow_ch_cnt = ch_cnt + 1; follow_ch_cnt < gtk_tree_model_iter_n_children (ks.ts_model, parent); ++follow_ch_cnt) {
                     GtkTreeIter iter_loop_follow_ch;
                     g_autofree gchar *menu_element_txt_loop_follow_ch;
 
@@ -178,15 +178,15 @@ void move_selection (gpointer direction_pointer)
             gtk_tree_store_move_before (ks.treestore, &ks.iter, NULL);
     }
 
-    push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+    push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
     activate_change_done ();
-    row_selected ();                     /* ... the toolbar buttons and 
-                                                application menu's undo/redo items get the correct sensitivity settings. */
+    row_selected ();                              /* ... the toolbar buttons and 
+                                                     application menu's undo/redo items get the correct sensitivity settings. */
 }
 
 /* 
 
-    Sets element visibility of menus, pipe menus, items and separators 
+    Sets element visibility of menus, pipe menus, items, and separators 
     that are dependent on the menu element that has been visualized.
 
 */
@@ -239,7 +239,7 @@ static gboolean check_and_adjust_dependent_element_visibilities (GtkTreeModel *f
         gtk_tree_path_compare (filter_path, filter_selected_path) == 0)) {
         gtk_tree_store_set (ks.treestore, &model_iter, TS_ELEMENT_VISIBILITY, "visible", -1);
         if (!menu_element_txt_filter && !STREQ (type_txt_filter, "separator")) {
-            gtk_tree_store_set (ks.treestore, &model_iter, TS_MENU_ELEMENT, "(Newly Created Label)",  -1);
+            gtk_tree_store_set (ks.treestore, &model_iter, TS_MENU_ELEMENT, _("(Newly Created Label)"),  -1);
         }
     }
     // Current row is a descendant of the selected row or not an ascendant of the selected row/the selected row itself.
@@ -253,7 +253,7 @@ static gboolean check_and_adjust_dependent_element_visibilities (GtkTreeModel *f
             ((STREQ (type_txt_filter, "item") ? "invisible item" : "invisible menu"))) : "visible";
 
         if (recursively_and_row_is_dsct && unlabeled_menu_or_item) {
-            gtk_tree_store_set (ks.treestore, &model_iter, TS_MENU_ELEMENT, "(Newly Created Label)", -1);
+            gtk_tree_store_set (ks.treestore, &model_iter, TS_MENU_ELEMENT, _("(Newly Created Label)"), -1);
         }
 
         gtk_tree_store_set (ks.treestore, &model_iter, TS_ELEMENT_VISIBILITY, new_element_visibility_txt, -1);
@@ -264,7 +264,7 @@ static gboolean check_and_adjust_dependent_element_visibilities (GtkTreeModel *f
 
 /* 
 
-    Changes the status of one or more menus, pipe menus, items and separators to visible.
+    Changes the status of one or more menus, pipe menus, items, and separators to visible.
 
 */
 
@@ -291,7 +291,7 @@ void visualize_menus_items_and_separators (gpointer recursively_pointer)
         gtk_tree_model_get (ks.ts_model, &iter_toplevel, TS_MENU_ELEMENT, &menu_element_txt_loop, -1);
 
         if (!menu_element_txt_loop) {
-            gtk_tree_store_set (ks.treestore, &iter_toplevel, TS_MENU_ELEMENT, "(Newly Created Label)",  -1);
+            gtk_tree_store_set (ks.treestore, &iter_toplevel, TS_MENU_ELEMENT, _("(Newly Created Label)"),  -1);
         }
 
         g_autoptr(GtkTreePath) path_toplevel = gtk_tree_model_get_path (ks.ts_model, &iter_toplevel);
@@ -336,10 +336,10 @@ void visualize_menus_items_and_separators (gpointer recursively_pointer)
         gtk_tree_path_free ((GtkTreePath *) filter_visualization[ED_FILTER_SELECTED_PATH]);
     }
 
-    push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+    push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
     activate_change_done ();
-    row_selected ();                     /* ... the toolbar buttons and 
-                                                application menu's undo/redo items get the correct sensitivity settings. */
+    row_selected ();                              /* ... the toolbar buttons and 
+                                                     application menu's undo/redo items get the correct sensitivity settings. */
 
 #if !(GLIB_CHECK_VERSION(2,56,0))
     // Cleanup
@@ -356,7 +356,7 @@ void visualize_menus_items_and_separators (gpointer recursively_pointer)
 static gboolean image_type_filter (              const GtkFileFilterInfo *filter_info, 
                                    G_GNUC_UNUSED       gpointer           user_data)
 {
-    return (regex_match_simple ("image/.*", filter_info->mime_type, PCRE2_UTF, 0));
+    return (g_regex_match_simple ("image/.*", filter_info->mime_type, 0, 0));
 }
 
 /* 
@@ -369,11 +369,11 @@ static gboolean image_type_filter (              const GtkFileFilterInfo *filter
 
 gchar *choose_icon (void)
 {
-    GtkWidget *dialog = gtk_file_chooser_dialog_new ("Open Image File",
+    GtkWidget *dialog = gtk_file_chooser_dialog_new (_("Open Image File"),
                                                      GTK_WINDOW (ks.window), 
                                                      GTK_FILE_CHOOSER_ACTION_OPEN,
-                                                     "_Cancel", GTK_RESPONSE_CANCEL, 
-                                                     "_Open", GTK_RESPONSE_ACCEPT, 
+                                                     C_("Cancel|File Dialogue", "_Cancel"), GTK_RESPONSE_CANCEL, 
+                                                     _("_Open"), GTK_RESPONSE_ACCEPT, 
                                                      NULL);
 
     GtkFileFilter *open_image_file_filter = gtk_file_filter_new ();
@@ -381,7 +381,7 @@ gchar *choose_icon (void)
 
     gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), "/");
 
-    gtk_file_filter_set_name (open_image_file_filter, "Image Files");
+    gtk_file_filter_set_name (open_image_file_filter, _("Image Files"));
     // The last two arguments (data and function to call to free data) aren't used.
     gtk_file_filter_add_custom (open_image_file_filter, GTK_FILE_FILTER_MIME_TYPE, (GtkFileFilterFunc) image_type_filter, 
                                 NULL, NULL);
@@ -415,10 +415,10 @@ void icon_choosing_by_button_or_context_menu (void)
 
     // The icon hasn't been chosen during the addition of a new (pipe) menu or item, so changes have been applied.
     if (!gtk_widget_get_visible (ks.enter_values_box)) {
-        push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+        push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
         activate_change_done ();
-        row_selected ();                     /* ... the toolbar buttons and 
-                                                    application menu's undo/redo items get the correct sensitivity settings. */
+        row_selected ();                              /* ... the toolbar buttons and 
+                                                         application menu's undo/redo items get the correct sensitivity settings. */
     }
 }
 
@@ -438,18 +438,42 @@ gboolean set_icon (      GtkTreeIter *icon_iter,
 
     if (G_UNLIKELY (!(icon_in_original_size = gdk_pixbuf_new_from_file (icon_path, &error)))) {
         if (display_err_msg) {
-            g_autoptr(GString) err_msg = g_string_new ("The following error occurred while trying to ");
+            g_autoptr(GString) err_msg = g_string_new (NULL);
 
             if (gtk_widget_get_visible (ks.remove_icon)) { // When entry fields are displayed at the bottom.
-                g_string_append_printf (err_msg, "add an icon to the %s '%s'", 
-                                        ks.txt_fields[TYPE_TXT], ks.txt_fields[MENU_ELEMENT_TXT]);
+                g_autofree gchar *err_msg_substr;
+
+                if (STREQ (ks.txt_fields[TYPE_TXT], "menu")) {
+                    err_msg_substr = g_strdup_printf (_("The following error occurred while trying to add an icon to the menu '%s'"), 
+                                                      ks.txt_fields[MENU_ELEMENT_TXT]);
+                }
+                else if (STREQ (ks.txt_fields[TYPE_TXT], "pipe menu")) {
+                    err_msg_substr = g_strdup_printf (_("The following error occurred while trying to add an icon to the pipe menu '%s'"), 
+                                                      ks.txt_fields[MENU_ELEMENT_TXT]);
+                }
+                else { // item
+                    err_msg_substr = g_strdup_printf (_("The following error occurred while trying to add an icon to the item '%s'"), 
+                                                      ks.txt_fields[MENU_ELEMENT_TXT]);
+                }
+
+                g_string_append (err_msg, err_msg_substr);
             }
             else { // When entry fields are displayed at the top (during the entry of a new menu element).
-                g_string_append_printf (err_msg, "add an icon to the new %s",   
-                                        (gtk_widget_get_visible (ks.entry_fields[EXECUTE_ENTRY])) ? 
-                                        "pipe menu" : (gtk_widget_get_visible (ks.entry_fields[MENU_ID_ENTRY])) ? 
-                                        "menu" : "item");
+                const gchar *err_msg_substr;
+
+                if (gtk_widget_get_visible (ks.entry_fields[EXECUTE_ENTRY])) {
+                    err_msg_substr = _("The following error occurred while trying to add an icon to the new pipe menu");
+                }
+                else if (gtk_widget_get_visible (ks.entry_fields[MENU_ID_ENTRY])) {
+                    err_msg_substr = _("The following error occurred while trying to add an icon to the new menu");
+                }
+                else { // item
+                    err_msg_substr = _("The following error occurred while trying to add an icon to the new item");
+                }
+
+                g_string_append (err_msg, err_msg_substr);
             }
+
             g_string_append_printf (err_msg, ":\n\n<span foreground='%s'>%s</span>", ks.red_hue, error->message);
 
             show_errmsg (err_msg->str);
@@ -523,7 +547,7 @@ void remove_icons_from_menus_or_items (gpointer push_on_undo_stack)
     }
 
     if (GPOINTER_TO_UINT (push_on_undo_stack)) {
-        push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+        push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
     }
     activate_change_done ();
     row_selected ();                         /* ... the toolbar buttons and 
@@ -581,13 +605,13 @@ void change_row (gpointer changed_entry_field_ptr)
             }
             if (streq_any (ks.txt_fields[TYPE_TXT], "menu", "pipe menu", NULL)) {
                 if (STREQ (ks.txt_fields[TYPE_TXT], "pipe menu") && !*entry_txt[EXECUTE_ENTRY]) {
-                    show_errmsg ("Pipe menus need a command to execute.");
+                    show_errmsg (_("Pipe menus need a command to execute."));
                     set_entry_fields ();
                     return;
                 }
                 if (!STREQ (ks.txt_fields[MENU_ID_TXT], entry_txt[MENU_ID_ENTRY])) {
                     if (G_UNLIKELY (g_slist_find_custom (ks.menu_ids, entry_txt[MENU_ID_ENTRY], (GCompareFunc) strcmp))) {
-                        show_errmsg ("This menu ID already exists. Please choose another one.");
+                        show_errmsg (_("This menu ID already exists. Please choose another one."));
                         set_entry_fields ();
                         return;
                     }
@@ -625,10 +649,10 @@ void change_row (gpointer changed_entry_field_ptr)
         gtk_tree_store_set (ks.treestore, &ks.iter, TS_VALUE, entry_txt[MENU_ELEMENT_OR_VALUE_ENTRY], -1);
     }
 
-    push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+    push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
     activate_change_done ();
-    row_selected ();                     /* ... the toolbar buttons and 
-                                                application menu's undo/redo items get the correct sensitivity settings. */
+    row_selected ();                              /* ... the toolbar buttons and 
+                                                     application menu's undo/redo items get the correct sensitivity settings. */
 }
 
 /* 
@@ -679,14 +703,14 @@ void cell_edited (G_GNUC_UNUSED       GtkCellRendererText *renderer,
     }
     else if (column_number == COL_MENU_ID) {
         if (G_UNLIKELY (g_slist_find_custom (ks.menu_ids, new_text, (GCompareFunc) strcmp))) {
-            show_errmsg ("This menu ID already exists. Please choose another one.");
+            show_errmsg (_("This menu ID already exists. Please choose another one."));
             return;
         }
         remove_menu_id (ks.txt_fields[MENU_ID_TXT]);
         ks.menu_ids = g_slist_prepend (ks.menu_ids, g_strdup (new_text));
     }
-    else if (column_number == COL_EXECUTE && !(*new_text)) {
-        show_errmsg ("Pipe menus need a command to execute.");
+    else if (G_UNLIKELY (column_number == COL_EXECUTE && !(*new_text))) {
+        show_errmsg (_("Pipe menus need a command to execute."));
         return;
     }
 
@@ -695,10 +719,10 @@ void cell_edited (G_GNUC_UNUSED       GtkCellRendererText *renderer,
         set_entry_fields ();
     }
 
-    push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+    push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
     activate_change_done ();
-    row_selected ();                     /* ... the toolbar buttons and 
-                                                application menu's undo/redo items get the correct sensitivity settings. */
+    row_selected ();                              /* ... the toolbar buttons and 
+                                                     application menu's undo/redo items get the correct sensitivity settings. */
 }
                                                                         
 /*
@@ -875,7 +899,7 @@ gboolean entry_completion_func (                    GtkEntryCompletion *entry_co
 
     const gchar *key_entry = gtk_entry_get_text (GTK_ENTRY (gtk_entry_completion_get_entry (entry_completion)));
 
-    return (regex_match_simple (key_entry, current_str, PCRE2_UTF | PCRE2_CASELESS, PCRE2_ANCHORED));
+    return (g_regex_match_simple (key_entry, current_str, G_REGEX_CASELESS, G_REGEX_MATCH_ANCHORED));
 }
 
 /*
@@ -904,14 +928,24 @@ gboolean focus_no_longer_on_entry_field (G_GNUC_UNUSED gpointer user_data)
 
 static void empty_label_msg (void)
 {
-    g_autofree gchar *label_txt = g_strdup_printf ("<b>The creation of an empty label for this %s is blocked, "
-                                                   "since it would make the %s invisible.</b>\n\n"
-                                                   "Existing menus with invisible menu elements may be opened; "
-                                                   "these menu elements can be visualized or deleted at once, "
-                                                   "otherwise they are highlighted.", 
-                                                   ks.txt_fields[TYPE_TXT], ks.txt_fields[TYPE_TXT]);
+    const gchar *msg_substr; 
 
-    show_message_dialog ("Entry of Empty Label Blocked", "dialog-error", label_txt, "_Close");
+    if (STREQ (ks.txt_fields[TYPE_TXT], "menu")) {
+        msg_substr = _("The creation of an empty label for this menu is blocked, since it would make the menu invisible.");
+    }
+    else if (STREQ (ks.txt_fields[TYPE_TXT], "pipe menu")) {
+        msg_substr = _("The creation of an empty label for this pipe menu is blocked, since it would make the pipe menu invisible.");
+    }
+    else { // item
+        msg_substr = _("The creation of an empty label for this item is blocked, since it would make the item invisible.");
+    }
+
+    g_autofree gchar *label_txt = g_strdup_printf ("<b>%s</b>\n\n%s", msg_substr, 
+                                                   _("Existing menus with invisible menu elements may be opened; "
+                                                     "these menu elements can be visualized or deleted at once, "
+                                                     "otherwise they are highlighted."));
+
+    show_message_dialog (_("Entry of Empty Label Blocked"), "dialog-error", label_txt, _("_Close"));
 }
 
 /* 
@@ -925,8 +959,8 @@ void boolean_toggled (void)
     gtk_tree_store_set (GTK_TREE_STORE (ks.ts_model), &ks.iter, TS_VALUE, 
                         (STREQ (ks.txt_fields[VALUE_TXT], "yes")) ? "no" : "yes", -1);
 
-    push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+    push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
     activate_change_done ();
-    row_selected ();                     /* ... the toolbar buttons and 
-                                                application menu's undo/redo items get the correct sensitivity settings. */
+    row_selected ();                              /* ... the toolbar buttons and 
+                                                     application menu's undo/redo items get the correct sensitivity settings. */
 }
